
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% VSBBO.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [x,f,info] = VSBBO(fun,x,st,tune);
%
% solve global unconstrained black box optimization problem 
%    min f(x) 
%  
% fun      % function handle for f(.)
% x        % starting point (must be specified)
% st       % structure with stop and print criteria
%          % (indefinite run if no stopping criterion is given)
%  .secmax       %   stop if sec>=secmax (default: inf)
%  .nfmax        %   stop if nf>=nfmax   (default: inf)
%  .ftarget      %   stop if f<=ftarget  (default: -inf)
%  .prt          %   printlevel (default: -1)
%                %   -1: nothing, 0: litte, >=1: more and more
% tune     % optional structure containing tuning parameters
%          %   for details see below
%
% x        % best point found 
% f        % function value at best point found 
% info     % structure with performance info
%          %   for details see below
% 
function [x,f,info] = VSBBO(fun,x,st,tune);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% check function handle
if isempty(fun)
  message = 'SBBO needs the function handle fun to be defined';
  disp(message)  
  return
elseif ~isa(fun,'function_handle')
  message = 'fun should be a function handle';
  disp(message)
  return
end


% initialize structure with information about points and function values
%
% point  % structure with information about points and function values
%  .x        %   point with best function value found
%  .f        %   best function value at x
%  .X        %   a list of the m best point so far
%  .F        %   the function values of X
%  .xl       %   point at xr-2p
%  .fl       %   function value at xr-2p
%  .xr       %   newest point 
%  .fr       %   function value at xr
%  .xm       %   point at xr-p
%  .fm       %   function value at xm 
%  .xinit    %   initial point
%  .finit    %   initial function value
%  .b        %   index of best point
%  .m        %   number of best point kept
%
% starting point
if isempty(x)
  message = 'starting point must be defined';
  disp(message) 
  return      
elseif ~isa(x,'numeric')
  message = 'x should be a numeric vector'; 
  disp(message ) 
  return       
end
% dimension
point.n = length(x);
% artificial lower and upper bounds to help prevent overflow
point.low=x-1e20;
point.upp=x+1e20;
point.x=x;
% other point info
point.X     = point.x; point.F  = NaN;   point.xinit = point.x; 
point.finit = NaN;     point.fr = NaN;   point.fm    = NaN;
point.fl    = NaN;     point.b  = NaN;   point.m     = 0; 
point.xr    = NaN+ones(point.n,1); point.xm = NaN+ones(point.n,1); 


% initialize structure containing all tuning parameters 
%
% tune % structure containing all tuning parameters 
%      % all parameters have a default that can be overwritten 
%      % by specifying it as an input
%  .mmax      % maximum number of best points kept
%  .T0        % maximal number of MLS in setScale
%  .H         % maximal number of heuristic direction
%  .S         % maximal number of subspace directions
%  .R         % maximal number of random directions per subspace
%             % direction in MLS
%  .E         % maximal number of extrapolations in MLS
%  .secSub    % scale subspace direction ?
%  .secCum    % scale cumulative direction ?
%  .cum       % select cumulative step type
%  .a         % upper bound to estimate cumulative step-size
%  .Deltamin  % minimium threshold for good impovement
%  .Deltamax  % initial threshold for good impovement
%  .deltainit % initial delta
%  .gamma1    % factor for finding delta
%  .gamma2    % factor for adjusting Deltamax
%  .gamma3    % factor for extrapolation test
%  .gamma4    % factor for finding initial lam
%  .gamma5    % factor for subspace direction
%  .gamma6    % factor for adjusting deltamin
%  .gamma7    % factor for adjusting deltamax
%  .gamma8    % factor for heuristic step
%
if ~exist('tune'), tune=[]; end;
% maximum number of best points kept
if ~isfield(tune,'mmax'), tune.mmax = 3; end;
tune.mmax=max(tune.mmax,3);
% maximal number of multi-line searches in setScales
if ~isfield(tune,'T0'), tune.T0 = 15; end;
tune.T0=max(tune.mmax,tune.T0);
% maximal number of heuristic trials
if ~isfield(tune,'H'), tune.H = 10; end;
% maximal number of subspace directions in multi-line search
if ~isfield(tune,'S'), tune.S = 2; end;
% number of random direction per subspace direction
if ~isfield(tune,'R'), tune.R = 10; end;
% maximal number of random trials
if ~isfield(tune,'E'), tune.E= 50; end;
%
% scale for subspace direction
if ~isfield(tune,'scSub'), tune.scSub = 0; end;
% scale for cumulative direction
if ~isfield(tune,'scCum'), tune.scCum = 0; end;
% select cumulative step type
if ~isfield(tune,'cum'), tune.cum = 2; end;
%
% upper bound for estimating cumulative step-size
if ~isfield(tune,'a'), tune.a = 1; end;
% minimium threshold for good impovement
if ~isfield(tune,'Deltamin'), tune.Deltamin = 0;end;
% initial threshold for good impovement
if ~isfield(tune,'Deltamax'), tune.Deltamax = 0;end;
% initial trial step
if ~isfield(tune,'deltainit'), tune.deltainit =  0.001; end;
% factor for finding delta
if ~isfield(tune,'gamma1'), tune.gamma1 = 2; end;
% factor for adjusting Deltamax
if ~isfield(tune,'gamma2'), tune.gamma2 = 1e-2; end;
% factor for extrapolation test
if ~isfield(tune,'gamma3'), tune.gamma3 = 2; end;
% factor for finding initial lam
if ~isfield(tune,'gamma4'), tune.gamma4 = 1e-3; end;
% factor for adjusting subspace direction
if ~isfield(tune,'gamma5'), tune.gamma5 = 1; end 
% factor for adjusting deltamin
if ~isfield(tune,'gamma6'), tune.gamma6 = 1; end;
% factor for adjusting deltamax
if ~isfield(tune,'gamma7'), tune.gamma7 = 10; end;
% factor for adjusting heuristic
if ~isfield(tune,'gamma8'), tune.gamma8 = 5000; end;


% initialize structure for step management
%
% step   % structure for step management
%  .s        % scaling vector
%  .p        % random search direction
%  .dp       % scaled length of p
%  .deltamin % minimum norm of trial steps
%  .deltamax % maximum norm of tial steps
%  .delta    % norm of trial steps 
%  .Delta    % threshold for good impovement
%  .Deltamin % minimium threshold for good impovement
%  .Deltamax % initial threshold for good impovement
%  .q        % cumulative step
%  .r        % cumulative gain
%
% initial gain required for a good step
step.Deltamin = tune.Deltamin;
% maximal gain required for a good step
step.Deltamax = tune.Deltamax;
% others
onesn = ones(point.n,1); 
step.r = NaN; step.dp = NaN; step.Delta = NaN;
step.delta = NaN; step.s = onesn; 
step.q= NaN+onesn; step.p = NaN+onesn; 


% par  % structure containing parameters modified during the search
%  .T        % maximal number of random directions
%  .lam      % lower bound on the Lipschitz constant
%  .good     % indicator for improvement
%  .ss       % are we in setScale?
%  .dir      % select direction type
%  .state    % state of cumulative step
%  .hs       % parameter for heuristic direction
%  .nmax     % parameter for heuristic direction
%
par.lam = NaN;   par.state = NaN;  par.dir = NaN; par.T = NaN; 
par.ss  = NaN;   par.good  = NaN; 


% info  % performance information for SBBO
%  .prt          % printlevel 
%                %   -1: nothing, 0: litte, >=1: more and more
%  .secmax       % stop if sec>=secmax 
%  .nfmax        % stop if nf>=nfmax 
%  .ftarget      % stop if f<=ftarget 
%  .initTime     % inital cputime
%  .done         % done with the search?
%  .nf           % number of function evaluations used
%  .nstep        % structure counting steps of a particular kind
%    .nH         %   number of heuristic direction   
%    .nR         %   number of scaled Random direction   
%    .nE         %   number of extrapolations
%    .nO         %   number of opposite direction
%    .nC         %   number of cumulative step
%    .nS         %   number of subspace step
%  .nstuck       % number of stucks
%  .sstate       % state of iterations  
%  .laststate    % state of last iteration
%  .ngood        % number of times that good holds in FDS
%  .nDelta       % number of times that Delta reduces in SBBO
%
% print level
if isfield(st,'prt'), info.prt = st.prt; 
else                  info.prt = -1; 
end;
% stopping criteria
if isfield(st,'secmax'), info.secmax=st.secmax;
else                     info.secmax=inf;
end;
if isfield(st,'nfmax'), info.nfmax=st.nfmax;
else                    info.nfmax=inf;
end;
if isfield(st,'ftarget'), info.ftarget=st.ftarget;
else                      info.ftarget=-inf;
end;
info.initTime=cputime;
% other
info.nstep.nH = 0;  % number of heuristic direction
info.nstep.nR = 0;  % number of scaled Random direction
info.nstep.nS = 0;  % number of subspace direction
info.nstep.nE = 0;  % number of extrapolate
info.nstep.nO = 0;  % number of opposite direction
info.nstep.nC = 0;  % number of cumulative step
info.nstuck   = 0;  % number of stuck
info.nDelta   = 0;  % number of times that Delta reduces in SBBO

for round=1, 
  % one round only, needed to be ableto jump to the end
  % when a stopping test is satisfied

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %                      get scales
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if info.prt==0,
    disp('======================')
    disp('start setScale')
    disp('======================')
  end
  [point,step,par,info] = setScales(fun,point,step,par,tune,info);
  if info.prt==0,
  disp('======================')
  disp('end setScale')
  disp('======================')
  end
  % check stopping test
  if info.done, break; end;
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %                      fixed decrease search
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if info.prt==0,
    disp('======================')
    disp('start FDS')
    disp('======================')
  end
  while 1
    [point,info] = FDS(fun,point,step,par,tune,info);
     % check stopping test
    if info.done, break; end;
    % gain of Delta unlikely with step of lenghth delta
    step.Delta = step.Delta/4; % reduce Delta
    info.nDelta = info.nDelta+1;
  end
end;

% assign results
x = point.X(:,point.b);
f = point.F(point.b);

if info.prt==0,
  disp('======================')
  disp('end SBBO')
  disp('======================')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

